@@ -170,7 +170,9 @@ class IndentFilter(ConcreteFilter):
 
     def __init__(self, indent):
         super(IndentFilter, self).__init__()
-        if isinstance(indent, int):
+        if indent is None:
+            self.indent = self.default
+        elif isinstance(indent, int):
             self.indent = self.default * indent
         else:
             self.indent = indent
@@ -181,12 +183,11 @@ class IndentFilter(ConcreteFilter):
 
 class StringFilter(ConcreteFilter):
 
-    """A filter that takes a translation string (256 characters) and
-    filters any incoming data through it."""
+    """A filter that takes a lookup table (e.g., string, list, or
+    dictionary) and filters any incoming data through it via
+    `str.translate`."""
 
     def __init__(self, table):
-        if not (isinstance(table, (em.bytes, em.str)) and len(table) == 256):
-            raise em.FilterError("table must be 256-character string")
         super(StringFilter, self).__init__()
         self.table = table
 
@@ -395,6 +396,9 @@ class Hook(AbstractHook):
 
     def preWhitespace(self, whitespace): pass
 
+    def preEnable(self, comment): pass
+    def preDisable(self, comment): pass
+
     def prePrefix(self): pass
 
     def preString(self, string): pass
@@ -439,8 +443,11 @@ class Hook(AbstractHook):
     def preEmoji(self, name): pass
     def postEmoji(self): pass
 
-    def preCustom(self, contents): pass
-    def postCustom(self, result): pass
+    def preExtension(self, name, contents, depth, locals): pass
+    def postExtension(self, result): pass
+
+    def preCustom(self, contents): pass # DEPRECATED
+    def postCustom(self, result): pass # DEPRECATED
 
     # Interpreter actions
 
@@ -462,6 +469,9 @@ class Hook(AbstractHook):
     def beforeFileFull(self, file, locals, dispatcher): pass
     def afterFileFull(self): pass
 
+    def beforeImport(self, filename, module, locals, dispatcher): pass
+    def afterImport(self): pass
+
     def beforeString(self, string, locals, dispatcher): pass
     def afterString(self): pass
 
@@ -477,8 +487,8 @@ class Hook(AbstractHook):
     def beforeSignificate(self, key, value, locals): pass
     def afterSignificate(self): pass
 
-    def beforeCallback(self, contents): pass
-    def afterCallback(self, result): pass
+    def beforeCallback(self, contents): pass # DEPRECATED
+    def afterCallback(self, result): pass # DEPRECATED
 
     def beforeAtomic(self, name, value, locals): pass
     def afterAtomic(self): pass
@@ -486,9 +496,6 @@ class Hook(AbstractHook):
     def beforeMulti(self, names, values, locals): pass
     def afterMulti(self): pass
 
-    def beforeImport(self, name, locals): pass
-    def afterImport(self): pass
-
     def beforeClause(self, catch, locals): pass
     def afterClause(self, exception, variable): pass
 
@@ -588,7 +595,7 @@ class Processor(em.Root):
     def scan(self, basename, extensions=None):
         if extensions is None:
             extensions = self.defaultExtensions
-        if isinstance(extensions, (em.bytes, em.str)):
+        if isinstance(extensions, em.strType):
             extensions = [extensions]
         def _noCriteria(x):
             return True
@@ -665,6 +672,7 @@ class Details(em.Root):
         self._implementation = None
         self._version = None
         self._framework = None
+        self._context = None
 
     def __contains__(self, key):
         return key in self.data
@@ -730,12 +738,39 @@ class Details(em.Root):
         assert self._implementation is not None
         return self._implementation
 
+    def checkPythonImplementation(self, choices):
+        """Check the Python implementation against the sequence of
+        choices."""
+        return self.getPythonImplementation() in choices
+
     def getPythonVersion(self):
         """Get the version of Python.  Cached."""
         if self._version is None:
-            self._version = platform.python_version()
+            try:
+                self._version = platform.python_version()
+            except AttributeError:
+                # Stackless Python 2.4 raises an internal error when this is
+                # called since sys.version is in an unexpected format.
+                self._version = '.'.join([str(x) for x in sys.version_info[:3]])
+        assert self._version is not None
         return self._version
 
+    def checkPythonVersion(self, minimum, maximum=None, closed=True):
+        """Check whether or not this Python version is greater than or
+        equal to the minimum, and if present, less than or equal to the
+        maximum."""
+        version = sys.version_info[:3]
+        if version < minimum:
+            return False
+        if maximum is not None:
+            if closed:
+                test = version >= maximum
+            else:
+                test = version > maximum
+            if test:
+                return False
+        return True
+
     def getMachineType(self):
         """Get the machine name.  Cached."""
         if self._machine is None:
@@ -764,8 +799,41 @@ class Details(em.Root):
             elif implementation == 'IronPython':
                 from System import Environment
                 self._framework = '.NET', Environment.Version
+            else:
+                # CPython
+                try:
+                    fields = platform.python_compiler().split()
+                    pair = fields[:2]
+                    rest = fields[3:]
+                    self._framework = tuple(pair)
+                except AttributeError:
+                    # Stackless Python 2.4 raises an internal error when this
+                    # is called since sys.version is in an unexpected format.
+                    lines = sys.version.splitlines()
+                    compiler = lines[-1]
+                    if compiler.startswith('[') and compiler.endswith(']'):
+                        compiler = compiler[1:-1]
+                    self._framework = tuple(compiler.split(' ', 1))
+        assert self._framework is not None
         return self._framework
 
+    def getContext(self):
+        """Get the context (with possible version) that this
+        interpreter is running in as a string or None.  Note: This is
+        not always possible.  Cached."""
+        if self._context is None:
+            if 'pyrun_config' in sys.modules:
+                self._context = 'PyRun'
+            elif 'Stackless' in sys.version:
+                fields = sys.version.split(None, 3)
+                self._context = fields[1] + '/' + fields[2]
+            elif 'ActiveState' in sys.copyright:
+                self._context = 'ActiveState'
+            else:
+                self._context = ''
+            assert self._context is not None
+        return self._context
+
     # Details.
 
     def sanitize(self, suffix):
@@ -872,8 +940,7 @@ class Details(em.Root):
             #
             # - Jython 2.5 raises an internal AttributeError when accessing
             #   platform.architecture.
-            # - IronPython 2.7 raises an internal NotImplementedError when
-            #   calling platform.architecture.
+            # - IronPython 2.7 has not implemented platform.architecture.
             # - Some older versions of IronPython 2.7 raise an ImportError when
             #   calling platform.platform which is then bizarrely reported as a
             #   TypeError with no traceback by the executive.
@@ -895,6 +962,7 @@ class Details(em.Root):
         self.accumulate('basic', 'implementation', self, 'getPythonImplementation', ())
         self.accumulate('basic', 'version', self, 'getPythonVersion', ())
         self.accumulate('basic/framework', ('name', 'version'), self, 'getFramework', ())
+        self.accumulate('basic', 'context', self, 'getContext', ())
 
     def getPythonDetails(self):
         """Collect details about this interpreter.  Use the
@@ -932,8 +1000,7 @@ class Details(em.Root):
 
     def getPlatformDetails(self):
         """Collect details about this platform.  Use the
-        platform module if available when the flag is set to
-        true."""
+        platform module if available when the flag is set to true."""
         self.accumulate('platform/architecture', ('bits', 'linkage'), platform, 'architecture', ())
         self.accumulate('platform', 'machine', platform, 'machine', ())
         self.accumulate('platform', 'name', platform, 'platform', ())
@@ -1081,6 +1148,9 @@ class Details(em.Root):
         if level >= em.Version.INFO:
             write(", in %s/%s" % (
                 self.getPythonImplementation(), self.getPythonVersion()))
+            context = self.getContext()
+            if context:
+                write(", as %s" % context)
             write(", on %s (%s)" % (
                 self.getSystemName(), self.getOSName()))
             write(", with %s" % self.getMachineType())
